@@ -3,6 +3,10 @@
 //! run time.  Note that data is stored in the underlying byte buffers in native endian form, thus
 //! requesting typed data from a buffer on a platform with different endianness will produce
 //! undefined behavior.
+//!
+//! # Caveats
+//!
+//! `DataBuffer` doesn't support zero-sized types.
 
 pub extern crate reinterpret;
 
@@ -21,7 +25,7 @@ use num_traits::{NumCast, Zero, cast};
 
 pub mod macros;
 
-/// Buffer of plain old data (POD). The data is stored as an array of bytes (`Vec<u8>`).
+/// Buffer of plain old data. The data is stored as an array of bytes (`Vec<u8>`).
 /// `DataBuffer` keeps track of the type stored within via an explicit `TypeId` member. This allows
 /// one to hide the type from the compiler and check it only when necessary. It is particularly
 /// useful when the type of data is determined at runtime (e.g. when parsing numeric data).
@@ -29,40 +33,34 @@ pub mod macros;
 pub struct DataBuffer {
     /// Raw data stored as an array of bytes.
     data: Vec<u8>,
-    /// Number of type sized chunks in the buffer.
-    length: usize,
+    /// Number of bytes occupied by ana element of this buffer.
+    /// Note: We store this instead of length because it gives us the ability to get the type size
+    /// when the buffer is empty.
+    element_size: usize,
     /// Type encoding for hiding the type of data from the compiler.
-    type_id: TypeId,
+    element_type_id: TypeId,
 }
 
 impl DataBuffer {
-    /// Construct an empty `DataBuffer`.
-    #[inline]
-    pub fn new() -> Self {
-        DataBuffer {
-            data: Vec::new(),
-            length: 0,
-            type_id: TypeId::of::<()>(),
-        }
-    }
-
     /// Construct an empty `DataBuffer` with a specific type.
     #[inline]
     pub fn with_type<T: Any>() -> Self {
+        let element_size = size_of::<T>();
+        assert_ne!(element_size, 0, "DataBuffer doesn't support zero sized types.");
         DataBuffer {
             data: Vec::new(),
-            length: 0,
-            type_id: TypeId::of::<T>(),
+            element_size,
+            element_type_id: TypeId::of::<T>(),
         }
     }
 
-    /// Construct an empty `DataBuffer` with a specific type given by the type id.
+    /// Construct a `DataBuffer` with the same type as the given buffer without copying its data.
     #[inline]
-    pub fn with_type_id(id: TypeId) -> Self {
+    pub fn with_buffer_type(other: &DataBuffer) -> Self {
         DataBuffer {
             data: Vec::new(),
-            length: 0,
-            type_id: id,
+            element_size: other.element_size,
+            element_type_id: other.element_type_id,
         }
     }
 
@@ -70,20 +68,12 @@ impl DataBuffer {
     /// setting byte capacity use `with_byte_capacity`.
     #[inline]
     pub fn with_capacity<T: Any>(n: usize) -> Self {
+        let element_size = size_of::<T>();
+        assert_ne!(element_size, 0, "DataBuffer doesn't support zero sized types.");
         DataBuffer {
-            data: Vec::with_capacity(n*size_of::<T>()),
-            length: 0,
-            type_id: TypeId::of::<T>(),
-        }
-    }
-
-    /// Construct an empty `DataBuffer` with a capacity for a given number of bytes.
-    #[inline]
-    pub fn with_byte_capacity(n: usize) -> Self {
-        DataBuffer {
-            data: Vec::with_capacity(n),
-            length: 0,
-            type_id: TypeId::of::<()>(),
+            data: Vec::with_capacity(n*element_size),
+            element_size,
+            element_type_id: TypeId::of::<T>(),
         }
     }
 
@@ -101,9 +91,7 @@ impl DataBuffer {
     /// ```
     #[inline]
     pub fn with_size<T: Any + Clone>(n: usize, def: T) -> Self {
-        let mut vec = Vec::with_capacity(n);
-        vec.resize(n, def);
-        Self::from_vec(vec)
+        Self::from_vec(vec![def; n])
     }
 
     /// Construct a `DataBuffer` from a given `Vec<T>` reusing the space already allocated by the
@@ -120,11 +108,12 @@ impl DataBuffer {
     /// # }
     /// ```
     pub fn from_vec<T: Any>(mut vec: Vec<T>) -> Self {
-        let length = vec.len();
+        let element_size = size_of::<T>();
+        assert_ne!(element_size, 0, "DataBuffer doesn't support zero sized types.");
 
         let data = {
-            let len_in_bytes = length * size_of::<T>();
-            let capacity_in_bytes = vec.capacity() * size_of::<T>();
+            let len_in_bytes = vec.len() * element_size;
+            let capacity_in_bytes = vec.capacity() * element_size;
             let vec_ptr = vec.as_mut_ptr() as *mut u8;
 
             unsafe {
@@ -135,8 +124,8 @@ impl DataBuffer {
 
         DataBuffer {
             data,
-            length,
-            type_id: TypeId::of::<T>(),
+            element_size,
+            element_type_id: TypeId::of::<T>(),
         }
     }
 
@@ -151,22 +140,21 @@ impl DataBuffer {
     /// Copy data from a given slice into the current buffer.
     #[inline]
     pub fn copy_from_slice<T: Any + Copy>(&mut self, slice: &[T]) -> &mut Self {
-        let length = slice.len();
-        let bins = length * size_of::<T>();
+        let element_size = size_of::<T>();
+        assert_ne!(element_size, 0, "DataBuffer doesn't support zero sized types.");
+        let bins = slice.len() * element_size;
         let byte_slice = unsafe { slice::from_raw_parts(slice.as_ptr() as *const u8, bins) };
         self.data.resize(bins, 0);
         self.data.copy_from_slice(byte_slice);
-        self.length = length;
-        self.type_id = TypeId::of::<T>();
+        self.element_size = element_size;
+        self.element_type_id = TypeId::of::<T>();
         self
     }
 
-    /// Clear the data buffer and set length to zero.
+    /// Clear the data buffer without destroying its type information.
     #[inline]
     pub fn clear(&mut self) {
         self.data.clear();
-        self.length = 0;
-        self.type_id = TypeId::of::<()>();
     }
 
     /// Fill the current buffer with copies of the given value. The size of the buffer is left
@@ -233,13 +221,14 @@ impl DataBuffer {
     /// Get the `TypeId` of data stored within this buffer.
     #[inline]
     pub fn type_id(&self) -> TypeId {
-        self.type_id
+        self.element_type_id
     }
 
     /// Get the number of elements stored in this buffer.
     #[inline]
     pub fn len(&self) -> usize {
-        self.length
+        debug_assert_eq!(self.data.len() % self.element_size, 0);
+        self.data.len() / self.element_size // element_size is guaranteed to be strictly positive
     }
 
     /// Get the byte capacity of this buffer.
@@ -249,11 +238,8 @@ impl DataBuffer {
     }
 
     /// Get the size of the element type.
-    pub fn size_of_type(&self) -> usize {
-        // Check that the length of the buffer is a multiple of the interpreted vector.
-        // This must always be true because of how we build the `DataBuffer`.
-        debug_assert_eq!( self.data.len() % self.length, 0 );
-        self.data.len() / self.length
+    pub fn element_size(&self) -> usize {
+        self.element_size
     }
 
     /// Return an iterator to a slice representing typed data.
@@ -454,7 +440,7 @@ impl DataBuffer {
     /// transferring data from one place to another for a generic buffer.
     #[inline]
     pub fn byte_chunks<'a>(&'a self) -> impl Iterator<Item=&'a [u8]> + 'a {
-        let chunk_size = self.size_of_type();
+        let chunk_size = self.element_size();
         self.data.chunks(chunk_size)
     }
 
@@ -464,7 +450,7 @@ impl DataBuffer {
     /// underlying untyped bytes (e.g. bit twiddling).
     #[inline]
     pub fn byte_chunks_mut<'a>(&'a mut self) -> impl Iterator<Item=&'a mut [u8]> + 'a {
-        let chunk_size = self.size_of_type();
+        let chunk_size = self.element_size();
         self.data.chunks_mut(chunk_size)
     }
 
@@ -474,9 +460,8 @@ impl DataBuffer {
     /// Otherwise, `None` is returned, and the buffer remains unmodified.
     #[inline]
     pub fn push_bytes(&mut self, bytes: &[u8]) -> Option<&mut Self> {
-        if bytes.len() == self.size_of_type() { 
+        if bytes.len() == self.element_size() { 
             self.data.extend_from_slice(bytes);
-            self.length += 1;
             Some(self)
         } else {
             None
@@ -489,10 +474,9 @@ impl DataBuffer {
     /// Otherwise, `None` is returned and the buffer is unmodified.
     #[inline]
     pub fn extend_bytes(&mut self, bytes: &[u8]) -> Option<&mut Self> {
-        let size_of_type = self.size_of_type();
-        if bytes.len() % size_of_type == 0 { 
+        let element_size = self.element_size();
+        if bytes.len() % element_size == 0 { 
             self.data.extend_from_slice(bytes);
-            self.length += bytes.len() / size_of_type;
             Some(self)
         } else {
             None
@@ -581,12 +565,6 @@ mod tests {
     /// Test various ways to create a data buffer.
     #[test]
     fn initialization_test() {
-        // Empty untyped buffer.
-        let a = DataBuffer::new();
-        assert_eq!(a.len(), 0);
-        assert_eq!(a.bytes_ref().len(), 0);
-        assert_eq!(a.type_id(), TypeId::of::<()>());
-
         // Empty typed buffer.
         let a = DataBuffer::with_type::<f32>();
         assert_eq!(a.len(), 0);
@@ -594,10 +572,10 @@ mod tests {
         assert_eq!(a.type_id(), TypeId::of::<f32>());
 
         // Empty buffer typed by the given type id.
-        let a = DataBuffer::with_type_id(TypeId::of::<f64>());
-        assert_eq!(a.len(), 0);
-        assert_eq!(a.bytes_ref().len(), 0);
-        assert_eq!(a.type_id(), TypeId::of::<f64>());
+        let b = DataBuffer::with_buffer_type(&a);
+        assert_eq!(b.len(), 0);
+        assert_eq!(b.bytes_ref().len(), 0);
+        assert_eq!(b.type_id(), TypeId::of::<f32>());
 
         // Empty typed buffer with a given capacity.
         let a = DataBuffer::with_capacity::<f32>(4);
@@ -605,13 +583,45 @@ mod tests {
         assert_eq!(a.bytes_ref().len(), 0);
         assert_eq!(a.byte_capacity(), 4*size_of::<f32>());
         assert_eq!(a.type_id(), TypeId::of::<f32>());
+    }
 
-        // Empty untyped buffer with a given byte capacity.
-        let a = DataBuffer::with_byte_capacity(4);
-        assert_eq!(a.len(), 0);
-        assert_eq!(a.bytes_ref().len(), 0);
-        assert_eq!(a.byte_capacity(), 4);
-        assert_eq!(a.type_id(), TypeId::of::<()>());
+    #[test]
+    #[should_panic]
+    fn zero_size_with_type_test() {
+        let _a = DataBuffer::with_type::<()>();
+    }
+
+    #[test]
+    #[should_panic]
+    fn zero_size_with_capacity_test() {
+        let _a = DataBuffer::with_capacity::<()>(2);
+    }
+
+    #[test]
+    #[should_panic]
+    fn zero_size_from_vec_test() {
+        let _a = DataBuffer::from_vec(vec![(); 3]);
+    }
+
+    #[test]
+    #[should_panic]
+    fn zero_size_with_size_test() {
+        let _a = DataBuffer::with_size(3, ());
+    }
+
+    #[test]
+    #[should_panic]
+    fn zero_size_from_slice_test() {
+        let v = vec![(); 3];
+        let _a = DataBuffer::from_slice(&v);
+    }
+
+    #[test]
+    #[should_panic]
+    fn zero_size_copy_from_slice_test() {
+        let v = vec![(); 3];
+        let mut a = DataBuffer::with_size(0, 1i32);
+        a.copy_from_slice(&v);
     }
 
     #[test]
